﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

using VFECore.Abilities;
using AbilityDef = VFECore.Abilities.AbilityDef;

namespace Outland
{
    public class Hediff_ArcaneClass : Hediff_AbilitiesWithLevels
	{
        public float arcaneEnergy;

		public ArcaneClass arcaneClass;

        public override string Label => "Outland.ArcaneClassLabel".Translate(arcaneClass.ToString());

        public override void PostAdd(DamageInfo? dinfo)
		{
			base.PostAdd(dinfo);
			arcaneEnergy = pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax);
		}

		public void UseEnergy(float energyUsed)
		{
			arcaneEnergy -= energyUsed;
			arcaneEnergy = Mathf.Min(arcaneEnergy, pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax));
		}

		public bool SufficientEnergyPresent(float energyWanted)
		{
			return arcaneEnergy > energyWanted;
		}

		public override void Tick()
		{
			base.Tick();
			arcaneEnergy += pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyRecoveryRate) / 60f;
			arcaneEnergy = Mathf.Min(arcaneEnergy, pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax));
		}

		public override IEnumerable<Gizmo> DrawGizmos()
		{
			yield return new Gizmo_ArcaneEnergyStatus
			{
				arcaneHediff = this
			};
		}

        //public override bool SatisfiesConditionForAbility(AbilityDef abilityDef)
        //{
        //	int result;
        //	if (!base.SatisfiesConditionForAbility(abilityDef))
        //	{
        //		HediffWithLevelCombination requiredHediff = abilityDef.requiredHediff;
        //		result = (requiredHediff != null && requiredHediff.minimumLevel == 1 && level == 0 && pawn.health.hediffSet.HasHediff(OutlandDefOf.Outland_NaturalMage)) ? 1 : 0;
        //	}
        //	else
        //	{
        //		result = 1;
        //	}
        //	return (byte)result != 0;
        //}

        public override void GiveRandomAbilityAtLevelForClass(int? forLevel = null)
		{
			if (giveRandomAbilities)
			{
				forLevel = new int?(forLevel ?? level);
				CompAbilities comp = pawn.GetComp<CompAbilities>();
				List<AbilityDef> source = DefDatabase<AbilityDef>.AllDefsListForReading.Where(delegate (AbilityDef def)
				{
					if (!comp.HasAbility(def) && def.requiredHediff != null && def.requiredHediff.hediffDef == this.def)
					{
						if (def.requiredHediff.minimumLevel <= forLevel.GetValueOrDefault() & forLevel != null)
						{
                            if (arcaneClass.HasFlag(def.GetModExtension<AbilityExtension_Arcane>().arcaneClass))
							{
								return def.requiredTrait == null || pawn.story.traits.HasTrait(def.requiredTrait);
							}
						}
					}
					return false;
				}).ToList<AbilityDef>();
				IEnumerable<AbilityDef> source2 = source.Where(delegate (AbilityDef def)
				{
					int minimumLevel = def.requiredHediff.minimumLevel;
					return minimumLevel == forLevel.GetValueOrDefault() & forLevel != null;
				});
				AbilityDef abilityDef;
				if (!source2.TryRandomElement(out abilityDef))
				{
					abilityDef = source.RandomElement<AbilityDef>();
				}
				comp.GiveAbility(abilityDef);
			}
		}

        public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look(ref arcaneEnergy, "arcaneEnergy", 0f);
			Scribe_Values.Look(ref arcaneClass, "arcaneClass", ArcaneClass.None);
		}
	}
}
