﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;
using VFECore.Abilities;

namespace Outland
{
    public class Hediff_AbilitiesWithLevels : Hediff_Abilities
    {
        public int experienceInt = 0;
        public int experienceLastTick = -1;

        public float levelFloat = 0f;
        public int levelInt = 0;

        public int skillPoints = 0;
        public int spentSkillPoints = 0;

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref experienceInt, "experienceInt", 0);
            Scribe_Values.Look(ref experienceLastTick, "experienceLastTick", -1);
            Scribe_Values.Look(ref levelFloat, "levelFloat", 0f);
            Scribe_Values.Look(ref levelInt, "levelInt", 0);
            Scribe_Values.Look(ref skillPoints, "skillPoints", 0);
            Scribe_Values.Look(ref spentSkillPoints, "spentSkillPoints", 0);
        }

        public DefModExt_LevelingHediff modExt;

        public bool IsMaxLevel
        {
            get
            {
                if(levelInt >= modExt.experienceCurve.Last().y)
                {
                    return true;
                }
                return false;
            }
        }

        public float PercentToNextLevel
        {
            get
            {
                float result = 0;
                if(modExt != null)
                {
                    if (IsMaxLevel)
                    {
                        result = 1f;
                    }
                    else
                    {
                        float currentLevelEXP = modExt.experienceCurve.EvaluateInverted(levelInt);
                        float nextLevelEXP = modExt.experienceCurve.EvaluateInverted(levelInt + 1);
                        result = (experienceInt - currentLevelEXP) / nextLevelEXP;
                    }
                }
                return result;
            }
        }

        public int AvailableSkillPoints => skillPoints - spentSkillPoints;

        public int ExpToNextLevel
        {
            get
            {
                int result = 0;
                if(modExt != null)
                {
                    result = (Mathf.RoundToInt(modExt.experienceCurve.EvaluateInverted(levelInt + 1)) - experienceInt);
                }
                return result;
            }
        }

        public override void Tick()
        {
            base.Tick();

            if(modExt == null)
            {
                modExt = def.GetModExtension<DefModExt_LevelingHediff>();
            }

            if(experienceLastTick != experienceInt || experienceInt < 0)
            {
                CalcLevel();
                CalcSkillPoints();
            }
            experienceLastTick = experienceInt;
        }

        public void CalcLevel()
        {
            if(modExt == null)
            {
                return;
            }
            levelFloat = modExt.experienceCurve.Evaluate(experienceInt);
            levelInt = Mathf.FloorToInt(levelFloat);
            if(level != levelInt)
            {
                SetLevelTo(levelInt);
            }
        }

        public void CalcSkillPoints()
        {
            if (modExt == null)
            {
                return;
            }
            skillPoints = modExt.skillPointsPerLevel * (levelInt - 1);
        }

        public void ExpGain(int xp)
        {
            experienceInt += xp;
        }

        public void SpendSkillPoints(int points)
        {
            skillPoints += points;
        }

        public override void GiveRandomAbilityAtLevel(int? forLevel = null)
        {
            if(forLevel > 1)
            {
                return;
            }
            GiveRandomAbilityAtLevelForClass(forLevel);
        }

        public virtual void GiveRandomAbilityAtLevelForClass(int? forLevel = null)
        {

        }
    }
}
