﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

using HarmonyLib;

using O21Toolbox;
using System.IO;
using System.Reflection;

namespace Outland
{
    public class OutlandMod : Mod
    {
        public static OutlandMod mod;
        public static OutlandSettings settings;

        public Vector2 scrollPosition;

        internal static string VersionDir => Path.Combine(ModLister.GetActiveModWithIdentifier("Neronix17.Outland.Core").RootDir.FullName, "Version.txt");
        public static string CurrentVersion { get; private set; }

        public OutlandMod(ModContentPack content) : base(content)
        {
            mod = this;
            settings = GetSettings<OutlandSettings>();

            Version version = Assembly.GetExecutingAssembly().GetName().Version;
            CurrentVersion = $"{version.Major}.{version.Minor}.{version.Build}";

            LogUtil.LogMessage($"Version: {CurrentVersion} ::");

            File.WriteAllText(VersionDir, CurrentVersion);

            new Harmony("neronix17.outland.core").PatchAll();
        }

        public override string SettingsCategory() => "Outland - Core";

        public override void DoSettingsWindowContents(Rect inRect)
        {
            float secondStageHeight;
            Listing_Standard listingStandard = new Listing_Standard();
            listingStandard.Begin(inRect);
            listingStandard.Note("Some settings may require a game restart to take effect.", GameFont.Tiny);
            listingStandard.GapLine();
            listingStandard.Gap(48);
            secondStageHeight = listingStandard.CurHeight;
            listingStandard.End();

            inRect.yMin = secondStageHeight;
            Rect outRect = inRect.ContractedBy(10f);

            float scrollRectHeight = 600f;
            Rect rect = new Rect(0f, 0f, outRect.width - (scrollRectHeight > outRect.height ? 18f : 0), scrollRectHeight);
            Widgets.BeginScrollView(outRect, ref scrollPosition, rect, true);
            listingStandard = new Listing_Standard();
            listingStandard.Begin(rect);

            listingStandard.Label("Kalocite");
            listingStandard.GapLine();
            listingStandard.CheckboxEnhanced("Glowing Kalocite", "Adds a glow to Kalocite if enabled. Disabled by default as high numbers of glowers can cause performance issues for some. Requires game restart.", ref settings.kalociteGlow);
            listingStandard.CheckboxEnhanced("Natural Kalocite", "Controls whether or not Kalocite spawns naturally through biomes. Requires game restart.", ref settings.kalociteSpawnsNaturally);
            if (settings.kalociteSpawnsNaturally)
            {
                listingStandard.AddLabeledSlider("Kalocite Spawn Multiplier: " + settings.kalociteCommonality.ToStringPercent(), ref settings.kalociteCommonality, 0f, 2f, "Min: 0%", "Max: 200%", 0.01f);
            }
            listingStandard.GapLine();
            listingStandard.Gap();

            listingStandard.Label("Bones");
            listingStandard.GapLine();
            listingStandard.AddLabeledSlider("Harvest Multiplier: " + settings.boneFactor.ToStringPercent(), ref settings.boneFactor, 0f, 2f, "Min: 0%", "Max: 200%", 0.01f);
            listingStandard.GapLine();
            listingStandard.Gap();



            listingStandard.End();
            Widgets.EndScrollView();

            base.DoSettingsWindowContents(inRect);
        }
    }
}
