﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace Outland
{
    [StaticConstructorOnStartup]
    public static class OutlandStartup
    {
        private static readonly Dictionary<ThingDef, ThingRequestGroup> registeredUnconventionalShells = new Dictionary<ThingDef, ThingRequestGroup>();

        static OutlandStartup()
        {
            if (OutlandMod.settings.kalociteSpawnsNaturally)
            {
                AddKalociteToBiomes();
            }

            FindAndRegisterUnconventionalShellsToTurrets();

            //RegisterUnconventionalShells(OutlandDefOf.Outland_Mangonel, ThingRequestGroup.HaulableEver);
            //RegisterUnconventionalShells(OutlandDefOf.Outland_Ballista, ThingRequestGroup.HaulableEver);
            //if (ModLister.GetActiveModWithIdentifier("Neronix17.Outland.MindFlayers") != null)
            //{
            //    RegisterUnconventionalShells(DefDatabase<ThingDef>.GetNamed("Outland_Cannon"), ThingRequestGroup.HaulableEver);
            //}

            //AddAppearanceToFabricsAndLeathers();
        }

        public static void FindAndRegisterUnconventionalShellsToTurrets()
        {
            List<ThingDef> unconventionalTurrets = DefDatabase<ThingDef>.AllDefs.Where(t => t.HasModExtension<DefModExt_UnconventionalTurret>()).ToList();

            foreach (ThingDef thing in unconventionalTurrets)
            {
                RegisterUnconventionalShells(thing, ThingRequestGroup.HaulableEver);
            }
        }

        //public static void AddAppearanceToFabricsAndLeathers()
        //{
        //    // Fabrics
        //    foreach(ThingDef def in DefDatabase<ThingDef>.AllDefs.Where(d => d.stuffProps != null && !d.stuffProps.categories.NullOrEmpty() && d.stuffProps.categories.Contains(StuffCategoryDefOf.Fabric)))
        //    {
        //        def.stuffProps.appearance = OutlandDefOf.OutlandFabric;
        //    }
        //    // Leathers
        //    foreach (ThingDef def in DefDatabase<ThingDef>.AllDefs.Where(d => d.stuffProps != null && !d.stuffProps.categories.NullOrEmpty() && d.stuffProps.categories.Contains(StuffCategoryDefOf.Leathery)))
        //    {
        //        def.stuffProps.appearance = OutlandDefOf.OutlandFabric;
        //    }
        //}

        public static void RegisterUnconventionalShells(ThingDef def, ThingRequestGroup ammoGroup)
        {
            if (!registeredUnconventionalShells.ContainsKey(def))
            {
                registeredUnconventionalShells.Add(def, ammoGroup);
            }
        }

        public static bool TryGetShells(this Thing thing, out ThingRequestGroup group) => registeredUnconventionalShells.TryGetValue(thing.def, out group);

        public static void AddKalociteToBiomes()
        {
            List<BiomeDef> biomeDefs = new List<BiomeDef>();

            foreach (BiomeDef biomeDef in DefDatabase<BiomeDef>.AllDefs)
            {
                if(biomeDef != null) { biomeDefs.Add(biomeDef); }
            }

            for (int i = 0; i < biomeDefs.Count; i++)
            {
                float commonality = 0f;
                foreach(BiomePlantRecord biomePlant in biomeDefs[i].wildPlants)
                {
                    if(biomePlant != null)
                    {
                        commonality += biomePlant.commonality;
                    }
                }

                BiomePlantRecord kalociteRecord = new BiomePlantRecord();
                kalociteRecord.plant = OutlandDefOf.Outland_KalociteFormation;
                kalociteRecord.commonality = commonality / (100f / (OutlandMod.settings.kalociteCommonality));
                biomeDefs[i].wildPlants.Add(kalociteRecord);
            }
        }
    }
}
